@@ -1,10 +1,8 @@
 # encoding: utf-8
 
-"""
-Initialization module for python-pptx
-"""
+"""Initialization module for python-pptx package."""
 
-__version__ = '0.6.5'
+__version__ = '0.6.6'
 
 
 import pptx.exc as exceptions
@@ -19,6 +17,7 @@ from pptx.opc.package import PartFactory
 from pptx.parts.chart import ChartPart
 from pptx.parts.coreprops import CorePropertiesPart
 from pptx.parts.image import ImagePart
+from pptx.parts.media import MediaPart
 from pptx.parts.presentation import PresentationPart
 from pptx.parts.slide import (
     NotesMasterPart, NotesSlidePart, SlideLayoutPart, SlideMasterPart,
@@ -45,11 +44,21 @@ content_type_to_part_class_map = {
     CT.TIFF:                  ImagePart,
     CT.X_EMF:                 ImagePart,
     CT.X_WMF:                 ImagePart,
+    CT.ASF:                   MediaPart,
+    CT.AVI:                   MediaPart,
+    CT.MOV:                   MediaPart,
+    CT.MP4:                   MediaPart,
+    CT.MPG:                   MediaPart,
+    CT.MS_VIDEO:              MediaPart,
+    CT.SWF:                   MediaPart,
+    CT.VIDEO:                 MediaPart,
+    CT.WMV:                   MediaPart,
+    CT.X_MS_VIDEO:            MediaPart,
 }
 
 PartFactory.part_type_for.update(content_type_to_part_class_map)
 
 del (
-    ChartPart, CorePropertiesPart, ImagePart, SlidePart, SlideLayoutPart,
-    SlideMasterPart, PresentationPart, CT, PartFactory
+    ChartPart, CorePropertiesPart, ImagePart, MediaPart, SlidePart,
+    SlideLayoutPart, SlideMasterPart, PresentationPart, CT, PartFactory
 )
