@@ -11,14 +11,14 @@ class PythonPptxError(Exception):
     """Generic error class."""
 
 
-class NotXMLError(PythonPptxError):
-    """
-    Raised when an XML operation (such as parsing) is attempted on a binary
-    package item.
-    """
-
-
 class PackageNotFoundError(PythonPptxError):
     """
     Raised when a package cannot be found at the specified path.
     """
+
+
+class InvalidXmlError(PythonPptxError):
+    """
+    Raised when a value is encountered in the XML that is not valid according
+    to the schema.
+    """
